SEXP savvy_rust_balance_deviation__ffi(SEXP c_arg__r_sample, SEXP c_arg__r_prob, SEXP c_arg__r_data);
SEXP savvy_rust_balanced__ffi(SEXP c_arg__r_prob, SEXP c_arg__r_bal_data, SEXP c_arg__r_eps, SEXP c_arg__r_method);
SEXP savvy_rust_balanced_stratified__ffi(SEXP c_arg__r_prob, SEXP c_arg__r_bal_data, SEXP c_arg__r_strata, SEXP c_arg__r_eps, SEXP c_arg__r_method);
SEXP savvy_rust_distributionally_balanced_design__ffi(SEXP c_arg__r_sample_size, SEXP c_arg__r_data, SEXP c_arg__r_temp, SEXP c_arg__r_cooling, SEXP c_arg__r_iter, SEXP c_arg__r_spatial_init, SEXP c_arg__r_method);
SEXP savvy_rust_distributionally_balanced_design_iter__ffi(SEXP c_arg__r_sample_size, SEXP c_arg__r_data, SEXP c_arg__r_temp, SEXP c_arg__r_cooling, SEXP c_arg__r_spatial_init, SEXP c_arg__r_iter_to, SEXP c_arg__r_iter_by, SEXP c_arg__r_method);
SEXP savvy_rust_doubly_balanced__ffi(SEXP c_arg__r_prob, SEXP c_arg__r_data, SEXP c_arg__r_bal_data, SEXP c_arg__r_eps, SEXP c_arg__r_bucket_size, SEXP c_arg__r_method);
SEXP savvy_rust_doubly_balanced_stratified__ffi(SEXP c_arg__r_prob, SEXP c_arg__r_data, SEXP c_arg__r_bal_data, SEXP c_arg__r_strata, SEXP c_arg__r_eps, SEXP c_arg__r_bucket_size, SEXP c_arg__r_method);
SEXP savvy_rust_local_mean_variance__ffi(SEXP c_arg__r_values, SEXP c_arg__r_prob, SEXP c_arg__r_data, SEXP c_arg__r_neighbours);
SEXP savvy_rust_pips_from_values__ffi(SEXP c_arg__r_values, SEXP c_arg__r_sample_size);
SEXP savvy_rust_spatial_balance_measure__ffi(SEXP c_arg__r_sample, SEXP c_arg__r_prob, SEXP c_arg__r_data, SEXP c_arg__r_method);
SEXP savvy_rust_spatial_balance_measure_all__ffi(SEXP c_arg__r_sample, SEXP c_arg__r_prob, SEXP c_arg__r_data);
SEXP savvy_rust_spatial_balance_measure_all_equal__ffi(SEXP c_arg__r_sample, SEXP c_arg__r_sample_size, SEXP c_arg__r_data);
SEXP savvy_rust_spatial_balance_measure_equal__ffi(SEXP c_arg__r_sample, SEXP c_arg__r_sample_size, SEXP c_arg__r_data, SEXP c_arg__r_method);
SEXP savvy_rust_spatially_balanced__ffi(SEXP c_arg__r_prob, SEXP c_arg__r_data, SEXP c_arg__r_eps, SEXP c_arg__r_bucket_size, SEXP c_arg__r_method);
SEXP savvy_rust_spatially_balanced_hierarchical__ffi(SEXP c_arg__r_prob, SEXP c_arg__r_data, SEXP c_arg__r_sizes, SEXP c_arg__r_eps, SEXP c_arg__r_bucket_size, SEXP c_arg__r_method);
SEXP savvy_rust_unequal__ffi(SEXP c_arg__r_prob, SEXP c_arg__r_eps, SEXP c_arg__r_method, SEXP c_arg__r_max_iter, SEXP c_arg__r_sample_size);

